@@ -2,29 +2,31 @@
 
 # specclassify, A Python package for multi- or hyperspectral image classification.
 #
-# Copyright (C) 2019  Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
+# Copyright (C) 2019-2021
+# - Daniel Scheffler (GFZ Potsdam, daniel.scheffler@gfz-potsdam.de)
+# - Helmholtz Centre Potsdam - GFZ German Research Centre for Geosciences Potsdam,
+#   Germany (https://www.gfz-potsdam.de/)
 #
 # This software was developed within the context of the GeoMultiSens project funded
 # by the German Federal Ministry of Education and Research
 # (project grant code: 01 IS 14 010 A-C).
 #
-# This program is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation, either version 3 of the License, or (at your option) any
-# later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 #
-# This program is distributed in the hope that it will be useful, but WITHOUT
-# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
-# details.
+#   http://www.apache.org/licenses/LICENSE-2.0
 #
-# You should have received a copy of the GNU Lesser General Public License along
-# with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 """Base classes for specclassify."""
 
 import numpy as np
-from typing import Union, List  # noqa F401  # flake8 issue
+from typing import Union, List, Tuple  # noqa F401  # flake8 issue
 from multiprocessing import Pool
 from tqdm import tqdm
 from matplotlib import pyplot as plt
@@ -76,7 +78,8 @@ class _ImageClassifier(object):
 
         return tilepos, cmap, dists
 
-    def classify(self, image_cube, in_nodataVal=None, cmap_nodataVal=None, tiledims=(100, 100)):
+    def classify(self, image_cube, in_nodataVal=None, cmap_nodataVal=-9999, tiledims=(100, 100)):
+        # type: (Union[GeoArray, np.ndarray], int, int, Tuple[int, int]) -> GeoArray
         """Classify the image.
 
         :param image_cube:
@@ -85,16 +88,23 @@ class _ImageClassifier(object):
         :param tiledims:
         :return:
         """
+        if not isinstance(cmap_nodataVal, int):
+            raise TypeError(cmap_nodataVal, "Expected an integer.")
+
         self._cmap_nodataVal = cmap_nodataVal
 
         dtype_cmap = np.int16
         if cmap_nodataVal is not None and not np.can_cast(cmap_nodataVal, dtype_cmap):
             dtype_cmap = np.find_common_type(np.array(self.train_labels), np.array([cmap_nodataVal]))
 
-        image_cube_gA = GeoArray(image_cube, nodata=in_nodataVal)  # lazily read in tiles to save memory
+        # lazily read in tiles to save memory
+        image_cube_gA = GeoArray(image_cube, nodata=in_nodataVal)
 
         bounds_alltiles = get_array_tilebounds(image_cube_gA.shape, tiledims)
 
+        # run classification #
+        ######################
+
         print('Performing %s image classification...' % self.clf_name)
         if self.CPUs is None or self.CPUs > 1:
             with Pool(self.CPUs, initializer=_mp_initializer, initargs=(self.train_spectra, image_cube_gA)) as pool:
@@ -105,7 +115,8 @@ class _ImageClassifier(object):
             tiles_results = [self._predict_tilewise(bounds) for bounds in tqdm(bounds_alltiles)]
 
         # use a local variable to avoid pickling in multiprocessing
-        cmap_dist_shape = (image_cube_gA.rows, image_cube_gA.cols) if tiles_results[0][1].ndim == 2 else \
+        cmap_dist_shape = \
+            (image_cube_gA.rows, image_cube_gA.cols) if tiles_results[0][1].ndim == 2 else \
             (image_cube_gA.rows, image_cube_gA.cols, tiles_results[0][1].shape[2])
         cmap = GeoArray(np.empty(cmap_dist_shape, dtype=dtype_cmap), nodata=cmap_nodataVal)
         cmap.unclassified_val = None
@@ -114,9 +125,12 @@ class _ImageClassifier(object):
         for tile_res in tiles_results:
             ((rS, rE), (cS, cE)), tile_cm = tile_res[:2]
             cmap[rS: rE + 1, cS: cE + 1] = tile_cm
+
             if len(tile_res) == 3:
                 dist[rS: rE + 1, cS: cE + 1] = tile_res[2]
 
+        ######################
+
         self.cmap = cmap
         if len(tiles_results[0]) == 3:
             self._distance_metrics = dist
@@ -133,8 +147,9 @@ class _ImageClassifier(object):
         :param imdata:
         :return:
         """
-        if imdata.nodata is not None and self._cmap_nodataVal is not None:
+        if imdata.nodata is not None:
             mask_anynodata = np.any(imdata[:] == imdata.nodata, axis=2)
+
             cmap[mask_anynodata] = self._cmap_nodataVal
 
         return cmap
